@@ -1,4 +1,4 @@
-# Copyright 2015-2021 D.G. MacCarthy <https://dmaccarthy.github.io/sc8pr>
+# Copyright 2015-2023 D.G. MacCarthy <https://dmaccarthy.github.io/sc8pr>
 #
 # This file is part of "sc8pr".
 #
@@ -18,10 +18,13 @@
 
 from random import uniform, random
 from math import sqrt, tan, pi, sin, cos, hypot
-from sys import stderr
 import pygame
 from pygame.pixelarray import PixelArray
 from sc8pr.util import rgba, style
+try:
+    from pygame.surfarray import pixels_alpha
+except:
+    pixels_alpha = None
 
 pi2 = 2 * pi
 
@@ -68,10 +71,13 @@ class Remove(Effect):
     def apply(self, img, n=0):
         if n <= 0:
             if self._complete:
-                self._gr.bind(ondraw=lambda gr:gr.remove())
+                self._gr.bind(ondraw=self.finish)
             else: self._gr.effects = None
         return img
 
+    @staticmethod
+    def finish(gr, ev=None): gr.remove()
+
 
 class ReplaceColor(Effect):
     "Replace one color by another (non-animated)"
@@ -193,108 +199,10 @@ class Squash(Wipe):
         return img
 
 
-class MathEffect(Effect):
-    "Effect based on y < f(x) or y > f(x)"
-
-    def __init__(self, noise=0.15, fill=(0,0,0,0)):
-        self.fill = rgba(fill)
-        self.above = noise > 0
-        self.noise = abs(noise)
-
-    def eqn(self, x, n, size):
-        dh = self.noise
-        if not self.above: n = 1 - n
-        return size[1] * ((1 + dh) * n - uniform(0, dh)), self.above
-
-    def apply(self, img, n=0):
-        "Modify image based on equation provided"
-        if n >= 1: return img
-        srf, size = self.srfSize(img, True)
-        h = size[1]
-        pxa = PixelArray(srf)
-        x = 0
-        for pxCol in pxa:
-            y = self.eqn(x, n, size)
-            if type(y) is tuple: y, above = y
-            else: above = True
-            if type(y) is float: y = int(y)
-            if above:
-                if y < h - 1:
-                    if y < 0: y = 0
-                    pxCol[y:] = self.fill
-            elif y > 0:
-                if y > h: y = h
-                pxCol[:y] = self.fill
-            x += 1
-        return srf
-
-
-class WipeSlope(MathEffect):
-    "Wipe diagonally from any corner"
-
-    def __init__(self, slope=False, above=True, fill=(0,0,0,0)):
-        self.slope = slope
-        self.above = above
-        self.fill = rgba(fill)
-
-    def eqn(self, x, n, size):
-        w, h = size
-        k = self.slope
-        if type(k) is bool:
-            k = h / w * (1 if k else -1)
-        if not self.above: n = 1 - n
-        dy = k * w
-        if k < 0: b = n * (h - dy)
-        else: b = n * (h + dy) - dy
-        return k * x + b, self.above
-
-
-class PaintDrops(MathEffect):
-
-    def __init__(self, drops=64, fill=(0,0,0,0)):
-        self.fill = rgba(fill)
-        self.side = drops > 0
-        self.drops = [self.makeDrop() for i in range(abs(drops))]
-        n = sum([d[0] for d in self.drops])
-        for d in self.drops: d[0] /= n
-
-    def eqn(self, x, n, size):
-        "Calculate paint boundary"
-        if not self.side: n = 1 - n
-        w, h = size
-        y = 0
-        xc = 0
-        for d in self.drops:
-            r = d[0] * w / 2
-            R = 1.1 * r
-            xc += r
-            dx = abs(x - xc)
-            if dx <= R:
-                dy = sqrt(R * R - dx * dx)
-                Y = (h + R) * self.posn(n, *d[1:]) + dy - R
-                if Y > y: y = Y
-            xc += r
-        return y, self.side
-
-    def posn(self, n, t1, t2):
-        "Calculate drop position"
-        if n < t1: return 0
-        elif n > t2: return 1
-        return (n - t1) / (t2 - t1)
-
-    @staticmethod
-    def makeDrop():
-        "Create random diameter, start and end time"
-        t1 = uniform(0, 0.8)
-        t2 = uniform(t1 + 0.1, 1)
-        return [uniform(0.1, 1), min(t1, t2), max(t1, t2)]
-
-
 class Dissolve(Effect):
     "Replace pixels randomly by a specified or random color"
 
     def __init__(self, colors=(0,0,0,0), keepTransparent=True, alpha=True):
-#        if not colors: self.colors = alpha
         t = type(colors)
         if t is int:
             self.colors = [rgba(alpha) for i in range(colors)]
@@ -310,13 +218,20 @@ class Dissolve(Effect):
         "Apply pixel-by-pixel effect"
         if n >= 1: return img
         srf = self.srfSize(img)
-        self.alphaMask = srf.map_rgb((0,0,0,255))
-        pxa = PixelArray(srf)
-        x = 0
-        for pxCol in pxa:
-            for y in range(len(pxCol)):
-                pxCol[y] = self.pixel(n, x, y, pxCol[y])
-            x += 1
+        if pixels_alpha: # Use numpy/pixels_alpha
+            sa = pixels_alpha(srf)
+            h = len(sa[0])
+            for c in sa:
+                for r in range(h):
+                    if random() > n: c[r] = 0
+        else: # No numpy!
+            self.alphaMask = srf.map_rgb((0,0,0,255))
+            pxa = PixelArray(srf)
+            x = 0
+            for pxCol in pxa:
+                for y in range(len(pxCol)):
+                    pxCol[y] = self.pixel(n, x, y, pxCol[y])
+                x += 1
         return srf
 
     def pixel(self, n, x, y, c):
@@ -329,24 +244,6 @@ class Dissolve(Effect):
         return c[self.n]
 
 
-class ClockHand(MathEffect):
-
-    def __init__(self, clockwise=True, fill=(0,0,0,0)):
-        self.fill = rgba(fill)
-        self.cw = clockwise
-
-    def eqn(self, x, n, size):
-        if n <= 0: return 0
-        h = size[1]
-        y = h / 2
-        x -= size[0] / 2
-        if not self.cw: x = -x
-        if x < 0:
-            return 0 if n <= 0.5 else (y + x * tan((n - 0.75) * pi2), False)
-        else:
-            return h if n >= 0.5 else (y + x * tan((n - 0.25) * pi2))
-
-
 class Pixelate(Effect):
 
     def __init__(self, size=64, linear=True):
@@ -373,22 +270,176 @@ class Pixelate(Effect):
         return srf
 
 
-try:
-    from pygame.surfarray import pixels_alpha
-    
-    class FastDissolve(Effect):
-        """Dissolve to transparency only; uses pygame.surfarray for speed but requires numpy"""
-    
-        def apply(self, img, n):
-            if n >= 1: return img
-            srf = self.srfSize(img)
-            sa = pixels_alpha(srf)
-            h = len(sa[0])
-            for c in sa:
-                for r in range(h):
-                    if random() > n: c[r] = 0
-            return srf
+class MathEffect(Effect):
+    "Effect based on y < f(x) or y > f(x)"
 
-except:
-    FastDissolve = Dissolve
-    print("FastDissolve effect is unavailable; substituting Dissolve", file=stderr)
+    def __init__(self, noise=0.15, fill=(0,0,0,0)):
+        self.fill = rgba(fill)
+        self.above = noise > 0
+        noise = abs(noise / 2)
+        self.limits(-noise, noise)
+
+    def func(self, x, t): return self.A * uniform(-1, 1)
+
+    def limits(self, y0, y1):
+        self.avg = (y1 + y0) / 2
+        self.A = (y1 - y0) / 2
+        self.h = 2 * self.A + 1
+        self.top = y1
+
+    def apply(self, img, n=0):
+        "Modify image based on equation provided"
+        if n >= 1: return img
+        srf, size = self.srfSize(img, True)
+        w, h = size
+        pxa = PixelArray(srf)
+        x = 0
+        t = n if self.above else (1 - n)
+        try: mid = t * self.h - self.A
+        except: pass
+        for pxCol in pxa:
+            if hasattr(self, "eqn"):  # Let subclass do all calculations
+                y = self.eqn(x, n, size)
+            else:  # Graph a function (scaled to (0,1)) with rising offset
+                y = (h - 1) * (self.func(x / (w - 1), t) + mid - self.avg)
+            if type(y) is tuple: y, above = y
+            else: above = self.above #True
+            if type(y) is float: y = int(y)
+            if above:
+                if y < h - 1:
+                    if y < 0: y = 0
+                    pxCol[y:] = self.fill
+            elif y > 0:
+                if y > h: y = h
+                pxCol[:y] = self.fill
+            x += 1
+        return srf
+
+
+class Wedge(MathEffect):
+
+    def __init__(self, slope=None, vertex=0.5, fill=(0,0,0,0)):
+        self.above = vertex > 0
+        self.fill = rgba(fill)
+        self.vertex = v = abs(vertex)
+        if slope is None:
+            self.m = 1/v, -1/(1-v)
+            self.limits(0, 1)
+        else:
+            slope = abs(slope)
+            self.m = slope, -slope
+            a, b = self.m
+            self.limits(0, slope * max(v, 1 - v))
+#             print(a, b, self.avg, self.A)
+
+    def func(self, x, t):
+        v = self.vertex
+        a, b = self.m
+        y = a * x if x < v else b * (x - v) + a * v
+#         y = x/v if x < v else (x-1)/(v-1)
+        return y if self.above else 1-y
+
+
+class WaterWaves(MathEffect):
+    "Rising water effect with waves"
+
+    def __init__(self, *args, speed=0.60, fill=(0,0,0,0), above=False):
+        # speed is total displacement over transition as fraction of width
+        # arg = (amplitude, wavelength, shift) as fraction of width/height
+        # (0.03, -0.2, 0.1)... A is 3% of height, wavelength  is 20% of width, shift is 10% of width
+        if not args: args = 6, 0.10, 0.15      # 6 waves, total A around 10%, wavelength around 15%
+        if type(args[0]) in (float, int):
+            N, A, wl = args
+            args = [(A/N*uniform(0.5, 1.5), wl*uniform(0.5, 1.5) * (-1 if i%2 else 1)) for i in range(N)]
+        self.args = args = [list(a) for a in args]
+        for a in args: a[1] = pi2 / a[1]
+        self.sumA = sum(abs(a[0]) for a in args)
+        self.fill = rgba(fill)
+        self.above = above
+        self.speed = speed
+        A = self.sumA
+        self.limits(-A, A)
+
+    def func(self, x, t):
+        p = self.args
+        s = self.speed
+        f = lambda a: a[0] * sin(abs(a[1]) * ((x + (-s if a[1] < 0 else s) * t) - (a[2] if len(a) > 2 else 0)))
+        return sum(f(p[i]) for i in range(len(p)))
+
+    def copy(self, n=-1):
+        "Return the 'reversed' or copied wave"
+        args = [[a[0], n*pi2/a[1]] + list(a[2:]) for a in self.args]
+        return WaterWaves(*args, speed=self.speed, fill=self.fill, above=self.above)
+
+
+class WipeSlope(MathEffect):
+    "Wipe diagonally from any corner"
+
+    def __init__(self, slope=-1, above=True, fill=(0,0,0,0)):
+        self.slope = slope
+        self.above = above
+        self.fill = rgba(fill)
+        if slope >= 0: self.limits(0, slope)
+        else: self.limits(slope, 0)
+
+    def func(self, x, n): return self.slope * x
+
+
+class ClockHand(MathEffect):
+    above = True
+
+    def __init__(self, clockwise=True, fill=(0,0,0,0)):
+        self.fill = rgba(fill)
+        self.cw = clockwise
+
+    def eqn(self, x, n, size):
+        if n <= 0: return 0
+        h = size[1]
+        y = h / 2
+        x -= size[0] / 2
+        if not self.cw: x = -x
+        if x < 0:
+            return 0 if n <= 0.5 else (y + x * tan((n - 0.75) * pi2), False)
+        else:
+            return h if n >= 0.5 else (y + x * tan((n - 0.25) * pi2))
+
+
+class PaintDrops(MathEffect):
+
+    def __init__(self, drops=64, fill=(0,0,0,0)):
+        self.fill = rgba(fill)
+        self.above = drops > 0
+        self.drops = [self.makeDrop() for i in range(abs(drops))]
+        n = sum([d[0] for d in self.drops])
+        for d in self.drops: d[0] /= n
+
+    def eqn(self, x, n, size):
+        "Calculate paint boundary"
+        if not self.above: n = 1 - n
+        w, h = size
+        y = 0
+        xc = 0
+        for d in self.drops:
+            r = d[0] * w / 2
+            R = 1.1 * r
+            xc += r
+            dx = abs(x - xc)
+            if dx <= R:
+                dy = sqrt(R * R - dx * dx)
+                Y = (h + R) * self.posn(n, *d[1:]) + dy - R
+                if Y > y: y = Y
+            xc += r
+        return y, self.above
+
+    def posn(self, n, t1, t2):
+        "Calculate drop position"
+        if n < t1: return 0
+        elif n > t2: return 1
+        return (n - t1) / (t2 - t1)
+
+    @staticmethod
+    def makeDrop():
+        "Create random diameter, start and end time"
+        t1 = uniform(0, 0.8)
+        t2 = uniform(t1 + 0.1, 1)
+        return [uniform(0.1, 1), min(t1, t2), max(t1, t2)]
