@@ -1,4 +1,4 @@
-# Copyright 2015-2021 D.G. MacCarthy <http://dmaccarthy.github.io>
+# Copyright 2015-2023 D.G. MacCarthy <http://dmaccarthy.github.io>
 #
 # This file is part of "sc8pr".
 #
@@ -24,9 +24,8 @@ import pygame
 from pygame.constants import K_UP, K_DOWN, K_LEFT, K_RIGHT, K_SPACE
 from sc8pr import Image, Sketch
 from sc8pr.sprite import Sprite
-from sc8pr.util import sc8prData, logError, rgba, noise, divAlpha
-from sc8pr.geom import vec2d, delta, DEG, dist, positiveAngle, angleDifference,\
-    subtend
+from sc8pr.util import logError, rgba, noise, divAlpha
+from sc8pr.geom import vec2d, delta, DEG, dist, angleDifference, subtend # , positiveAngle
 from sc8pr.shape import Line, Polygon
 
 
@@ -46,9 +45,11 @@ class RobotThread(Thread):
             print('{} is running in thread {}.'.format(*args), file=stderr)
         try:
             while r._startup: r.sleep()
-            r._startTime = time()
+#             r._startTime = time()
+            r._uptime = 0
             r.brain()
             if hasattr(r, "shutdown"): r.shutdown()
+            r._uptime = None
         except: logError()
         if self.log:
             print('{} is shutting down in thread {}.'.format(*args), file=stderr)
@@ -80,7 +81,7 @@ class Robot(Sprite):
     proximity = None
 
     def __init__(self, colors=None):
-        img = Image.fromBytes(sc8prData("robot"))
+        img = Image.fromZip("robot")
         if colors:  # Replace body and nose colors
             px = pygame.PixelArray(img.image)
             body0, nose0, body, nose = rgba("red", "blue", *colors)
@@ -91,19 +92,20 @@ class Robot(Sprite):
                 nose0 = _tempColor(px, nose0, *colors)
             if nose != nose0: px.replace(nose0, nose)
             if body != body0: px.replace(body0, body)
-        img = img.tiles(2)
-        super().__init__(img)
+        super().__init__(img.tiles(2))
+        self._uptime = None
 
     @property
-    def uptime(self): return time() - self._startTime
+    def uptime(self): return self._uptime
+#         return time() - self._startTime
 
     @property
     def gyro(self):
-        return positiveAngle(self.angle - self._gyro)
+        return (self.angle - self._gyro) % 360
 
     @gyro.setter
     def gyro(self, g):
-        self._gyro = positiveAngle(g - self.angle)
+        self._gyro = (g - self.angle) % 360
 
     @property
     def stopped(self):
@@ -136,8 +138,12 @@ class Robot(Sprite):
     def sleep(self, t=None):
         "Sleep for the specified time"
         if not self.active: raise InactiveError()
-        if t: sleep(t)
-        else: sleep(1 / self.sketch.frameRate)
+        fTime = 1 / self.sketch.frameRate
+        if t:
+            t1 = self._uptime + t
+            while t1 - self.uptime > fTime:
+                sleep(t)
+        else: sleep(fTime)
 
     @property
     def motors(self): return self._motors
@@ -151,7 +157,7 @@ class Robot(Sprite):
     @property
     def power(self): return sum(abs(m) for m in self._motors) / 2
 
-    def onbounce(self, wall):
+    def onbounce(self, ev):
         "Adjust motion upon collision with wall"
         w, h = self.sketch.size
         x, y = self.pos
@@ -167,15 +173,19 @@ class Robot(Sprite):
         self.vel = vx, vy
         self.collision = True
 
-    def oncollide(self): self.collision = True
+    def oncollide(self, ev=None): self.collision = True
 
-    def ondraw(self):
+    def update(self, ev=None):
         "Update robot sprite each frame"
 
         if not self.active: raise InactiveError()
 
-        # Target wheel speed...
+        # Update timer
         sk = self.sketch
+        if self._uptime is not None:
+            self._uptime += 1 / sk.frameRate        
+
+        # Target wheel speed...
         v = self.maxSpeed * sk.width
         v1, v2 = self._motors
         v1 *= v
@@ -203,7 +213,7 @@ class Robot(Sprite):
         self.costumeTime = 0 if p == 0 else round(36 / (1 + 5 * p))
 
         # Update position and angle...
-        super().ondraw()
+        super().update(ev)
 
         # Update sensors if requested...
         if self._updateSensors:
@@ -281,7 +291,7 @@ class Robot(Sprite):
 
     def _drawLEDs(self):
         "Draw LEDs on the robot to indicate color sensor data"
-        for costume in self._costumes:
+        for costume in self.costumeList:
             srfs = costume._srf
             orig = srfs.original
             r0 = orig.get_width() / 2
