@@ -24,7 +24,7 @@ from sc8pr.geom import shiftAlongNormal, angleDifference, DEG
 from sc8pr.shape import Polygon, Line, Circle
 from sc8pr.util import rangef
 from sc8pr.robot.gui import Robot as _Robot
-from sc8pr.sprite import physics, Sprite
+from sc8pr.sprite import Sprite
 
 
 class Robot(_Robot):
@@ -38,8 +38,8 @@ class Robot(_Robot):
         self._gyro_data = []
         self._gyro_sample = t if t is None else round(self.sketch.frameRate * t)
 
-    def ondraw(self):
-        _Robot.ondraw(self)
+    def update(self, ev=None):
+        _Robot.update(self, ev)
         if self._gyro_sample: self.get_gyro_data()
 
     def get_gyro_data(self):
