@@ -1,4 +1,4 @@
-# Copyright 2015-2021 D.G. MacCarthy <http://dmaccarthy.github.io>
+# Copyright 2015-2023 D.G. MacCarthy <http://dmaccarthy.github.io>
 #
 # This file is part of "sc8pr".
 #
@@ -19,12 +19,13 @@
 from math import hypot
 from pygame.sprite import collide_mask, collide_rect, collide_circle
 from sc8pr import BaseSprite, Image, Graphic
+from sc8pr.util import customEv
 
 
 class CostumeImage(Graphic):
-    costumeTime = 0
-    _costumeNumber = 0
-    onreset = None
+    _seq = None
+    cycle = True
+    _costumeNumber = costumeTime = 0
 
     def __init__(self, image, cols=1, rows=1, flip=0, padding=0):
         # Clone costumes images from existing list
@@ -35,31 +36,38 @@ class CostumeImage(Graphic):
         else: tiles = Image(image).tiles(cols, rows, flip, padding)
 
         # Initialize
-        self._costumes = self.costumeList = tiles
+        self._costumeList = tiles
         self._size = tiles[0].size
 
-    def __len__(self): return len(self._costumes)
-    def __getitem__(self, i): return self._costumes[i]
+    def _index(self, i=None):
+        "Get the index in the ORIGINAL costumeList from the index in the CURRENT sequence"
+        if i is None: i = self._costumeNumber
+        return self._seq[i] if self._seq else i
+    
+    def __len__(self): return len(self._seq if self._seq else self._costumeList)
+    def __getitem__(self, i): return self._costumeList[self._index(i)]
 
     @property
     def costumeNumber(self): return self._costumeNumber
  
     @costumeNumber.setter
-    def costumeNumber(self, n):
-        self._costumeNumber = n % len(self._costumes)
+    def costumeNumber(self, n): self._costumeNumber = n % len(self)
 
     def costume(self):
         "Return an Image instance of the current costume"
-        n = self.costumeNumber
-        return self._costumes[n].config(size=self.size, angle=self.angle)
+        return self._costumeList[self._index()].config(size=self.size, angle=self.angle)
 
-    def costumeSequence(self, seq, n=None):
+    @property
+    def costumeList(self): return self._costumeList
+
+    @property
+    def costumeSequence(self): return self._seq
+
+    @costumeSequence.setter
+    def costumeSequence(self, seq):
         "Select and order the costumes to animate"
-        self._costumes = list(self.costumeList[i] for i in seq)
-        if n is None and self._costumeNumber >= len(self._costumes):
-            n = 0
-        if n is not None: self.costumeNumber = n
-        return self
+        self._seq = tuple(seq)
+        if self._costumeNumber >= len(self): n = 0
 
     @property
     def avgColor(self): return self.costume().avgColor
@@ -75,27 +83,41 @@ class CostumeImage(Graphic):
         img.rect = self.rect
         return img.contains(pos)
 
-    def updateCostume(self):
+    def updateCostume(self, ev):
         "Change sprite costume"
         n = self.costumeTime
         if n < 0:
             dn = -1
             n = -n
         else: dn = 1
-        if n and self.sketch.frameCount % n == 0:
-            self.costumeNumber = self._costumeNumber + dn
-            if self._costumeNumber == 0 and self.onreset: 
-                self.onreset()
+        try: f = self.sketch.frameCount
+        except: f = None
+        if n and f and f % n == 0:
+            cn = self._costumeNumber
+            x = cn + dn
+            if x in (len(self), -1):
+                if self.cycle: s = 2
+                else:
+                    x = cn
+                    s = -1
+                    self.costumeTime = 0
+            else: s = 1
+            if x != cn: self.costumeNumber = x
+        else: s = 0
+        setattr(ev, "status", s)
 
-    ondraw = updateCostume
+    def update(self, ev):
+        self.updateCostume(ev)
+        if hasattr(self, "ondraw"): self.ondraw(ev)
 
 
 class Sprite(CostumeImage, BaseSprite):
     "Sprite animation with one or more costumes"
 
-    def ondraw(self):
-        BaseSprite.ondraw(self)
-        self.updateCostume()
+    def update(self, ev):
+        self.motion()
+        self.updateCostume(ev)
+        if hasattr(self, "ondraw"): self.ondraw(ev)
 
 
 def collide_rect_mask(left, right):
